﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Pathfinding;

[RequireComponent (typeof (Rigidbody2D))]
[RequireComponent (typeof (Seeker))]
[RequireComponent (typeof (CollisionItem))]
[RequireComponent (typeof (Animator))]
[RequireComponent(typeof(AudioSource))]
public class EnemyBehaviour : MonoBehaviour {

    //speed
    public float speed = 10f;
    public float speedOfBullets = 2f;
    public ForceMode2D fMode;
    public float updateRate = 2f;
    public bool pathEnded = false;
    public float nextWpDist = 3;        //max dist from AI to waypoint
    private int currentWP = 0;
    
    // Check ceiling
    public Transform ceilingCheck;
    public float ceilingRadius = .01f;

    //Force
    private float forceX = -30f, forceY;
    public float bulletChance = 0.9f;

    private bool facingRight = false;

    public Transform shootPrefab, summonPrefab;
    public Path path;                   
    private Seeker seeker;
    private Animator anim;
    private CollisionItem colItem;
    private AnimatorClipInfo[] clipInfo;
    private Rigidbody2D rigidComp;      
    private GameObject target;          //Our Goblin target
    private Vector2 dir;
    private Vector2 lastChekpointPos;   //lastCheckpoint Position
    private bool updatePathTriggered = false;
    private bool createProjectileTriggered = false;
    private Camera cam;
    private AudioSource audio;
    private bool triggerSummon = false;

    private void Awake()
    {
        cam = GameObject.FindGameObjectWithTag("MainCamera").GetComponent<Camera>();
        audio = GetComponent<AudioSource>();
        colItem = GetComponent<CollisionItem>();
        colItem.enabled = false;
        anim = GetComponent<Animator>();
        clipInfo = anim.GetCurrentAnimatorClipInfo(0);
        rigidComp = GetComponent<Rigidbody2D>();
        target = GameObject.FindGameObjectWithTag("Player");
        seeker = GetComponent<Seeker>();
        lastChekpointPos = this.gameObject.transform.position;
    }

    private void Start()
    {
        
        if (this.gameObject.name.Contains("EyeMonster"))
        {
            updatePathTriggered = true;
            StartCoroutine(UpdatePath());
        }
        else if (this.gameObject.name.Contains("Summoned"))
        {
            StartCoroutine(JumpAttack());
        }

    }

    public void SaveLastCheckpointPos()
    {
        lastChekpointPos = gameObject.transform.position;
    }

    public void RestoreLastCheckpointPos()
    {
        this.gameObject.transform.position = lastChekpointPos;
    }

    IEnumerator UpdatePath()
    {
        if (updatePathTriggered)
        {
            //Start a path to the target pos
            seeker.StartPath(transform.position, target.transform.position, OnPathComplete);


            if (this.gameObject.name.Contains("EyeMonster") || this.gameObject.name.Contains("Summon"))
                yield return new WaitForSeconds(1f / Random.Range(0.5f, updateRate));
            else
                yield return new WaitForSeconds(1f / updateRate);
            StartCoroutine(UpdatePath());
        }
    }

    public void OnPathComplete(Path p)
    {
        //Debug.Log("Path found. Did it have an error? " + p.error);
        if(!p.error)
        {
            path = p;
            currentWP = 0;
        }
    }

	// Update is called once per frame
	void Update ()
    {
        if(AudioController.isFXActive())
        {
            if(!audio.isPlaying)
                audio.UnPause();
        }
        else
        {
            audio.Pause();
        }

        //Lets see if the target and the enemy are visible for the main camera
        if (AreObjectsVisible(target, this.gameObject, cam))
        {
            anim.SetTrigger("SeeAble");
        }
        else
        {
            anim.SetBool("SeeAble", false);
            //updatePathTriggered = false;
            rigidComp.velocity -= rigidComp.velocity;      //Stop inerness of the addforce
        }

        if ((this.gameObject.name.Contains("Sorcerer") || this.gameObject.name.Contains("Wizard") || this.gameObject.name.Contains("Summoner")) && anim.GetBool("SeeAble") && !updatePathTriggered)
        {
            updatePathTriggered = true;
            StartCoroutine(UpdatePath());
        }
        
        //Flipping depending on maincharacter position
        if ((target.transform.position.x > transform.position.x) && !facingRight)
            Flip();
        else if ((target.transform.position.x < transform.position.x) && facingRight)
            Flip();

        //CALCULATE PATH
        if (path == null)
            return;

        //End of the path
        if (currentWP >= path.vectorPath.Count)
        {
            if (pathEnded)
                return;

            //Debug.Log("End of path reached.");
            pathEnded = true;
            return;
        }
        pathEnded = false;

        CalculateDirection();

        if (anim.GetBool("SeeAble"))
        {
            rigidComp.AddForce(dir, fMode);  //Move the AI

            if (this.gameObject.name.Contains("EyeMonster") && !createProjectileTriggered)
            {
                createProjectileTriggered = true;
                StartCoroutine(CreateProjectile());              //Create a ball in direction of the character position
            }
            else if (this.gameObject.name.Contains("Sorcerer"))
            {
                //Check ceiling (if the character is near)
                if (IsTargetNear())
                {
                    anim.SetBool("CeilingCheck", true);
                    StartCoroutine(EnableColliderInSeconds(clipInfo[0].clip.length));
                    //It will appear, attack and dissapear when is not near
                }
                else
                {
                    anim.SetBool("CeilingCheck", false);
                }
            }
            else if (this.gameObject.name.Contains("Wizard"))
            {
                if (IsTargetNear())
                {
                    anim.SetBool("CeilingCheck", true);
                    StartCoroutine(FrontAttack());
                }
                else
                {
                    anim.SetBool("CeilingCheck", false);
                }
            }
            else if (this.gameObject.name.Contains("Summoner"))
            {
                if (AreObjectsVisible(target, this.gameObject, cam))
                {
                    if(!triggerSummon)
                    {
                        triggerSummon = true;
                        StartCoroutine(CreateSummon());
                    }
                }
                else
                {
                    triggerSummon = false;
                    StopCoroutine(CreateSummon());
                }
            }
        }
        else
        {
            anim.SetBool("SeeAble", false);
            if (this.gameObject.name.Contains("EyeMonster"))
            {
                Destroy(this.gameObject);
            }
        }

       CalculateDistance();
    }

    IEnumerator EnableColliderInSeconds(float seconds)
    {
        colItem.enabled = false;
        yield return new WaitForSeconds(seconds);
        colItem.enabled = true;
    }

    IEnumerator DisableColliderInSeconds(float seconds)
    {
        colItem.enabled = true;
        yield return new WaitForSeconds(seconds);
        colItem.enabled = false;
    }

    IEnumerator FrontAttack()
    {
        if(anim.GetBool("CeilingCheck"))
        {
            anim.SetBool("Attack2", true);
            rigidComp.velocity = Vector2.zero;
            yield return DisableColliderInSeconds(2);
            rigidComp.AddForce(dir,fMode);
            yield return BulletAttack();
        }
    }

    IEnumerator BulletAttack()
    {
        if (Random.value < bulletChance && anim.GetBool("Attack2"))
        {
            anim.SetBool("Attack", true);
            yield return new WaitForSeconds(clipInfo[0].clip.length);
            Vector2 myPos = new Vector2(transform.position.x, transform.position.y);
            Vector2 targetPos = new Vector2(target.transform.position.x, target.transform.position.y);
            Vector2 direction = (targetPos - myPos).normalized;
            Quaternion rotation = Quaternion.Euler(0, 0, Mathf.Atan2(direction.y, direction.x) * Mathf.Rad2Deg);

            Transform projectile = Instantiate(shootPrefab, myPos, rotation);
            projectile.GetComponent<Rigidbody2D>().velocity = direction * speedOfBullets;
            yield return new WaitForSeconds(clipInfo[0].clip.length);
            StartCoroutine(BulletAttack());
        }
        else
        {
            anim.SetBool("Attack2", false);
            anim.SetBool("Attack", false);
            StopCoroutine(BulletAttack());
        }
    }

    IEnumerator CreateProjectile()
    {

        if (anim.GetBool("SeeAble") && createProjectileTriggered)
        {
            if (gameObject.name.Contains("EyeMonster"))
            {
                yield return new WaitForSeconds(3);
            }
            Vector2 myPos = new Vector2(transform.position.x, transform.position.y);
            Vector2 targetPos = new Vector2(target.transform.position.x, target.transform.position.y);
            Vector2 direction = (targetPos - myPos).normalized;
            Quaternion rotation = Quaternion.Euler(0, 0, Mathf.Atan2(direction.y, direction.x) * Mathf.Rad2Deg);

            Transform projectile = Instantiate(shootPrefab, myPos, rotation);
            projectile.GetComponent<Rigidbody2D>().velocity = direction * speedOfBullets;
            StartCoroutine(CreateProjectile());
        }
        else
            StopCoroutine(CreateProjectile());
    }

    bool AreObjectsVisible(GameObject target, GameObject target2, Camera cam)
    {
        return target.GetComponent<SpriteRenderer>().IsVisibleFrom(cam) && target2.GetComponent<SpriteRenderer>().IsVisibleFrom(cam);
    }

    bool IsTargetNear()
    {
        Collider2D[] hitColliders = Physics2D.OverlapCircleAll(ceilingCheck.position, ceilingRadius);
        int i = 0;
        if(hitColliders!=null)
        {
            while (i < hitColliders.Length)
            {
                if (hitColliders[i].tag == "Player")
                {
                    return true;
                }
                i++;
            }
        }
        return false;
    }
    
    IEnumerator CreateSummon()
    {
        anim.SetBool("Idle", false);
        float secs = clipInfo[0].clip.length; //length of anim
        Vector2 myPos = new Vector2(transform.position.x, transform.position.y);
        Vector2 targetPos = new Vector2(target.transform.position.x, target.transform.position.y);
        Vector2 direction = (targetPos - myPos).normalized;
        Quaternion rotation = Quaternion.Euler(direction.x, 0, 0);
        yield return new WaitForSeconds(secs);
        var hitColliders = Physics.OverlapSphere(myPos, 1);//1 is purely chosen arbitrarly
        if (hitColliders.Length == 0)
            Instantiate(summonPrefab, myPos, rotation);
        anim.SetBool("Idle", true);
        yield return new WaitForSeconds(secs);
        StartCoroutine(CreateSummon());
        
    }

    IEnumerator JumpAttack()
    {
        yield return new WaitForSeconds(Random.Range(1, 2));
        forceY = Random.Range(150, 200);
        rigidComp.AddForce(new Vector2(forceX, forceY));
        yield return new WaitForSeconds(1f);
        StartCoroutine(JumpAttack());
    }

    
    void CalculateDirection()
    {
        dir = (path.vectorPath[currentWP] - transform.position).normalized;
        dir *= speed * Time.fixedDeltaTime;
    }
    
    void CalculatePath()
    {
        if (path == null)
            return;

        //End of the path
        if (currentWP >= path.vectorPath.Count)
        {
            if (pathEnded)
                return;

            //Debug.Log("End of path reached.");
            pathEnded = true;
            return;
        }
        pathEnded = false;

        CalculateDirection();
        CalculateDistance();
    }

    void CalculateDistance()
    {
        float dist = Vector3.Distance(transform.position, path.vectorPath[currentWP]);
        if (dist < nextWpDist)
        {
            currentWP++;
            return;
        }
    }

    void Flip()
    {
        facingRight = !facingRight;
        Vector3 theScale = transform.localScale;
        theScale.x *= -1;
        transform.localScale = theScale;
        forceX *= -1;
    }
}
