﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;
using UnityEngine.SceneManagement;

[RequireComponent (typeof(Animator))]
[RequireComponent (typeof(AudioSource))]
[RequireComponent (typeof(Collider2D))]
[RequireComponent (typeof(DropRandomItem))]
public class DamageableItem : MonoBehaviour {
    public float currentHealth;
    public int startingHealth = 100;            // The amount of health the enemy starts the game with.
    public float sinkSpeed = 2.5f;              // The speed at which the items sinks through the floor when dead.
    public int scoreValue = 10;                 // The amount added to the player's score when the enemy dies.
    public AudioClip deathClip, hurtClip;       // Sounds

    Animator anim;                              // Reference to the animator.
    AudioSource audio;                          // Reference to the audio source.
    Collider2D collider;                        // Reference to the collider.
    Rigidbody2D rb;
    DropRandomItem droppables;
    EnemyBehaviour eb;
    Pathfinding.Seeker seek;
    CollisionItem colItem;
    public bool isDead;                                // Whether the item is dead.
    public bool damageTaken = false;
    private bool invulnerability = false;
    private float lastCheckpointHealth;
    public bool Invulnerability
    {
        get
        {
            return invulnerability;
        }

        set
        {
            invulnerability = value;
        }
    }

    void Awake ()
    {
        // Setting up the references.
        anim = GetComponent <Animator> ();
        audio = GetComponent <AudioSource> ();
        collider = GetComponent <Collider2D> ();
        droppables = GetComponent<DropRandomItem>();
        rb = GetComponent<Rigidbody2D>();
        eb = GetComponent<EnemyBehaviour>();
        seek = GetComponent<Pathfinding.Seeker>();
        colItem = GetComponent<CollisionItem>();
        // Setting the current health 
        currentHealth = startingHealth;

        //If it is the player, we will control the life bar
        if (this.gameObject.CompareTag("Player"))
            HealthBarScript.health = startingHealth;
        else
        {
            if (PlayerPrefs.GetFloat("enemyHealthProportion")<0)
                startingHealth = Mathf.RoundToInt(startingHealth * PlayerPrefs.GetFloat("enemyHealthProportion"));
        }

        FloatingTextController.Initialize();

    }

    private void Start()
    {
        SaveLastHealth();
    }

    public void TakeDamage (int amount)
    {
        damageTaken = true;
        // If the item is dead...
        if(isDead)
            // ... no need to take damage so exit the function.
            return;

        // Play the hurt sound effect.
        if (AudioController.isFXActive())
        {
            audio.PlayOneShot(hurtClip);
        }

        // Reduce the current health by the amount of damage sustained.
        currentHealth -= amount;

        // Create floating text
        FloatingTextController.CreateFloatingText(amount.ToString(), this.gameObject.transform, Color.red);

        //Update health bar of the player
        if (this.gameObject.CompareTag("Player"))
            HealthBarScript.health -= amount;

        //Update Score with the damage done
        if (this.gameObject.CompareTag("Enemy"))
            ScoreManager.newPoint(amount);

        // If the current health is less than or equal to zero...
        if (currentHealth <= 0)
        {
            // ... the item is dead.
            if (this.gameObject.name.Contains("Summoned"))
            {
                Destroy(this.gameObject);
            }
            Death ();
        }
    }

    public void RandomHeal()
    {
        float amount = Random.Range(20, 50);
        Heal(amount);
    }

    public void Heal(float amount)
    {
        if (currentHealth + amount > startingHealth) //Limiting max heal
        {
            amount = (currentHealth + amount) - startingHealth;
            currentHealth = startingHealth;
        }
        else
            currentHealth += amount;

        HealthBarScript.health += amount;
        FloatingTextController.CreateFloatingText(amount.ToString(), this.gameObject.transform, Color.green);
    }

    private void Update()
    {
        //If the object is the player is going down, then cause death
        if (this.gameObject.CompareTag("Player"))
        {
            GoblinControllerScript gc = gameObject.GetComponent<GoblinControllerScript>();
            if (gc.CurrentFallTime >= gc.maxFallTime)
            {
                Death();
                gc.CurrentFallTime = 0;
            }
        }
    }    

    void Death ()
    {

        // Change the audio clip of the audio source to the death clip and play it (this will stop the hurt clip playing).
        if (AudioController.isFXActive())
        {
            audio.PlayOneShot(deathClip);
        }

        if (this.gameObject.CompareTag("Player"))
        {
            //We will reset points, we will only keep the taken crystals
            ScoreManager.resetPoints();
            LifeScript.lifes -= 1;
            if(LifeScript.lifes==0)
            {
                SceneLoader.LoadGameOverMenu();
            }
            else
            {
                PlayerPrefs.SetInt("lifes", LifeScript.lifes);
                SceneLoader.LoadRetryMenu();
            }
        }
        else
        {
            // Dropping items
            droppables.DropRandomItemsWithChance();

            isDead = true;
            // Turn the collider into a trigger so shots can pass through it.
            collider.isTrigger = true;
            rb.constraints = RigidbodyConstraints2D.FreezeRotation | RigidbodyConstraints2D.FreezePositionX;     //Sinking without rotating
            rb.gravityScale = 0.3f;                                     //Gravity for falling to the ground
            Destroy(eb);
            Destroy(seek);
            Destroy(colItem);
            Destroy(gameObject, 3f);                                    //After seconds the gameObject will be destroyed
            anim.SetTrigger("Dead");                                    //Triggering the death animation

            if (this.gameObject.name.Contains("Wizard"))
            {
                GameObject door = GameObject.FindGameObjectWithTag("Door");
                door.transform.GetChild(0).gameObject.SetActive(true);
                door.transform.GetChild(1).gameObject.SetActive(false);

                LifeScript.lifes += 1;
            }
        }
    }

    public void SaveLastHealth()
    {
        lastCheckpointHealth = currentHealth;
        if (this.gameObject.CompareTag("Player"))
            HealthBarScript.health = currentHealth;
    }

    public void RestoreLastHealth()
    {
        currentHealth = lastCheckpointHealth;
        if (this.gameObject.CompareTag("Player"))
        {
            HealthBarScript.health = currentHealth;
        }
    }
}
